==> ldas-tools-al-2.5.0.tar.gz: ldas-tools-al-2.5.0/test/tLDASUnexpected.cc <==
#include <setjmp.h>
#include <unistd.h>

#include <new>
#include <typeinfo>

#include "ldastoolsal/unittest.h"

#include "ldastoolsal/unexpected_exception.hh"
#include "ldastoolsal/SignalHandler.hh"

// Only need to include this file so we can turn off logging
#include "LDASUnexpected.hh"

using LDASTools::AL::SignalHandler;
using LDASTools::Error::unexpected_exception;
using LDASTools::Error::LDASUnexpected;

typedef LDASTools::AL::SignalHandler::signal_type signal_type;

#if HAVE_RETHROW_WORKING_IN_BAD_EXCEPTION && ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
#define	FULL_TEST 1
#endif

LDASTools::Testing::UnitTest Test;

static jmp_buf	jmpbuffer;

static class Abort_
  : public SignalHandler::Callback
{
public:
  typedef SignalHandler::signal_type signal_type;
  
  virtual void
  SignalCallback( signal_type Signal )
  {
    std::cerr << "DEBUG: Abort handler was called."
	      << std::endl
      ;
    longjmp(jmpbuffer, 1);
  }
} abort_handler;

// Test with no exception specification
void TestValid001_func()
{
#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
  throw std::runtime_error("Thrown from TestValid001");
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
} 
void TestValid001()
{
#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
  try {
    TestValid001_func();
  }
  catch(std::runtime_error& e)
  {
    Test.Check(true) << "Caught runtime_error(\""
		     << e.what() << "\")" << std::endl;
  }
  catch(...)
  {
    Test.Check(false) << "Caught wrong exception" << std::endl;
  }
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
}

// Test with correct exception specification
void
TestValid002_func()
  throw(std::runtime_error)
{
  throw std::runtime_error("Thrown from TestValid002");
}

void
TestValid002()
{
  static const char* caller = "TestValid002";
  std::ostringstream	msg;
  bool pass = false;

  try {
    TestValid002_func( );
  }
  catch(std::runtime_error& e)
  {
    pass = true;
    msg << "Caught runtime_error(\""
	<< e.what() << "\")"
      ;
  }
  catch( const std::exception& e )
  {
    msg << "Caught wrong exception: "
	<< e.what( )
      ;
  }
  catch(...)
  {
    msg << "Caught wrong exception";
  }
  Test.Check( pass )
    << caller << ": " << msg.str( )
    << std::endl
    ;
}

// Test with correct exception specification but invalid throw
void
TestValid003_func()
    throw(std::invalid_argument, unexpected_exception)
{
    throw std::runtime_error("Thrown from TestValid003");
}

void
TestValid003()
{
#if FULL_TEST
  static const char* caller = "TestValid003";
  std::ostringstream	msg;
  bool pass = false;

  try {
    TestValid003_func();
  }
  catch( const unexpected_exception& e)
  {
    pass = true;
    msg << "Caught unexpected_exception(\""
	<< e.what() << "\", \""
	<< e.msg() << "\")"
      ;
  }
  catch( const std::exception& e)
  {
    msg << "Caught wrong exception(\""
	<< e.what() << "\""
	<< ")"
      ;
  }
  catch(...)
  {
    msg << "Caught wrong exception";
  }
  Test.Check( pass )
    << caller << ": " << msg.str( )
    << std::endl
    ;
#endif /* FULL_TEST */
}

// Test with incorrect exception specification
template<class E>
void ThrowUnexpected(const std::string& what)
    throw(unexpected_exception)
{
    throw E(what);
}

template<class E>
void ThrowUnexpected()
    throw(unexpected_exception)
{
    throw E();
}

template<class E>
void TestValid(const std::string& name, const std::string& what)
{
    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	ThrowUnexpected<E>(what);
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
	
	Test.Check(std::string(e.msg()) == what)
	  << "What()s match - " << e.msg() << " == " << what << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
	Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

template<class E>
void TestValid(const std::string& name)
{
    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
#if defined( FULL_TEST )
	ThrowUnexpected<E>();
#else /* FULL_TEST */
	//---------------------------------------------------------------
	// This ugly piece of code is needed because of an apparent
	//   problem with gcc 3.0 which generates an ABORT if this is
	//   not done.
	//---------------------------------------------------------------
	try {
	  throw E();
	}
#define	CATCH(EX) \
	catch( EX & e ) \
	{ \
	  ThrowUnexpected<EX>(); \
	}
	CATCH(std::bad_alloc)
	CATCH(std::bad_exception)
	CATCH(std::bad_typeid)
	CATCH(std::bad_cast)
	CATCH(std::exception)
#undef CATCH
	catch(...)
	{
	  Test.Check(false) << "Unknown exception throw: " << name
			    << std::endl;
	}
#endif /* FULL_TEST */
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
        Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

void TestValid_bad_alloc( void )
{
    const std::string name("std::bad_alloc");

    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	ThrowUnexpected<std::bad_alloc>();
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
        Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

// Test with incorrect exception specification
void TestInvalid001()
    throw()
{
    throw std::runtime_error("Thrown from TestInvalid001");
}

int main(int argc, char** argv)
{
    Test.Init(argc, argv);

    LDASUnexpected::makeAbort( false );
    SignalHandler::Register( &abort_handler,
			     SignalHandler::SIGNAL_ABORT );

    if (!Test.IsVerbose())
    {
	LDASUnexpected::makeQuiet();
    }

    TestValid001();
    TestValid002();
    TestValid003();

#if FULL_TEST
#define TEST_VALID(exc)  TestValid<exc>(std::string(#exc), \
                                        std::string("Testing validity of " #exc))

#define TEST_VALID2(exc) TestValid<exc>(std::string(#exc))

    TEST_VALID(std::length_error);
    TEST_VALID(std::domain_error);
    TEST_VALID(std::out_of_range);
    TEST_VALID(std::invalid_argument);
    TEST_VALID(std::underflow_error);
    TEST_VALID(std::overflow_error);
    TEST_VALID(std::range_error);
    TEST_VALID(std::logic_error);
    TEST_VALID(std::runtime_error);

    TEST_VALID2(std::bad_alloc);
    TEST_VALID2(std::bad_exception);
    TEST_VALID2(std::bad_typeid);
    TEST_VALID2(std::bad_cast);
    TEST_VALID2(std::exception);

    TestValid_bad_alloc();
#endif	/* FULL_TEST */

#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
    // This test unavoidably causes an abort, as it should
    // Setup to be silent
    int err_dup( dup( 2 ) );
    close( 2 );

    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(true) << "TestInvalid001 - Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	TestInvalid001();
      }
      catch(...)
      {
	// TestInvalid001 should cause an abort - if we catch
	// an exception in this case there is a bug!
	Test.Check(false) << "Caught an exception!"
			 << std::endl;
      }
    }
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
    err_dup = dup( err_dup );

    Test.Exit();

    return 0;
}

==> ldas-tools-al-2.5.7.tar.gz: ldas-tools-al-2.5.7/test/tLDASUnexpected.cc <==
#include <setjmp.h>
#include <unistd.h>

#include <new>
#include <typeinfo>

#include "ldastoolsal/unittest.h"

#include "ldastoolsal/unexpected_exception.hh"
#include "ldastoolsal/SignalHandler.hh"

// Only need to include this file so we can turn off logging
#include "LDASUnexpected.hh"

using LDASTools::AL::SignalHandler;
using LDASTools::Error::unexpected_exception;
using LDASTools::Error::LDASUnexpected;

typedef LDASTools::AL::SignalHandler::signal_type signal_type;

#if HAVE_RETHROW_WORKING_IN_BAD_EXCEPTION && ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
#define	FULL_TEST 1
#endif

LDASTools::Testing::UnitTest Test;

static jmp_buf	jmpbuffer;

static class Abort_
  : public SignalHandler::Callback
{
public:
  typedef SignalHandler::signal_type signal_type;
  
  virtual void
  SignalCallback( signal_type Signal )
  {
    std::cerr << "DEBUG: Abort handler was called."
	      << std::endl
      ;
    longjmp(jmpbuffer, 1);
  }
} abort_handler;

// Test with no exception specification
void TestValid001_func()
{
#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
  throw std::runtime_error("Thrown from TestValid001");
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
} 
void TestValid001()
{
#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
  try {
    TestValid001_func();
  }
  catch(std::runtime_error& e)
  {
    Test.Check(true) << "Caught runtime_error(\""
		     << e.what() << "\")" << std::endl;
  }
  catch(...)
  {
    Test.Check(false) << "Caught wrong exception" << std::endl;
  }
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
}

// Test with correct exception specification
void
TestValid002_func()
{
  throw std::runtime_error("Thrown from TestValid002");
}

void
TestValid002()
{
  static const char* caller = "TestValid002";
  std::ostringstream	msg;
  bool pass = false;

  try {
    TestValid002_func( );
  }
  catch(std::runtime_error& e)
  {
    pass = true;
    msg << "Caught runtime_error(\""
	<< e.what() << "\")"
      ;
  }
  catch( const std::exception& e )
  {
    msg << "Caught wrong exception: "
	<< e.what( )
      ;
  }
  catch(...)
  {
    msg << "Caught wrong exception";
  }
  Test.Check( pass )
    << caller << ": " << msg.str( )
    << std::endl
    ;
}

#if UNUSED
// Test with correct exception specification but invalid throw
static void
TestValid003_func()
    throw(std::invalid_argument, unexpected_exception)
{
    throw std::runtime_error("Thrown from TestValid003");
}
#endif /* UNUSED */

void
TestValid003()
{
#if FULL_TEST
  static const char* caller = "TestValid003";
  std::ostringstream	msg;
  bool pass = false;

  try {
    TestValid003_func();
  }
  catch( const unexpected_exception& e)
  {
    pass = true;
    msg << "Caught unexpected_exception(\""
	<< e.what() << "\", \""
	<< e.msg() << "\")"
      ;
  }
  catch( const std::exception& e)
  {
    msg << "Caught wrong exception(\""
	<< e.what() << "\""
	<< ")"
      ;
  }
  catch(...)
  {
    msg << "Caught wrong exception";
  }
  Test.Check( pass )
    << caller << ": " << msg.str( )
    << std::endl
    ;
#endif /* FULL_TEST */
}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated"
// Test with incorrect exception specification
template<class E>
static void ThrowUnexpected(const std::string& what)
    throw(unexpected_exception)
{
    throw E(what);
}
#pragma GCC diagnostic pop

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated"
template<class E>
static void ThrowUnexpected()
    throw(unexpected_exception)
{
    throw E();
}
#pragma GCC diagnostic pop

template<class E>
void TestValid(const std::string& name, const std::string& what)
{
    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	ThrowUnexpected<E>(what);
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
	
	Test.Check(std::string(e.msg()) == what)
	  << "What()s match - " << e.msg() << " == " << what << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
	Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

template<class E>
void TestValid(const std::string& name)
{
    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
#if defined( FULL_TEST )
	ThrowUnexpected<E>();
#else /* FULL_TEST */
	//---------------------------------------------------------------
	// This ugly piece of code is needed because of an apparent
	//   problem with gcc 3.0 which generates an ABORT if this is
	//   not done.
	//---------------------------------------------------------------
	try {
	  throw E();
	}
#define	CATCH(EX) \
	catch( EX & e ) \
	{ \
	  ThrowUnexpected<EX>(); \
	}
	CATCH(std::bad_alloc)
	CATCH(std::bad_exception)
	CATCH(std::bad_typeid)
	CATCH(std::bad_cast)
	CATCH(std::exception)
#undef CATCH
	catch(...)
	{
	  Test.Check(false) << "Unknown exception throw: " << name
			    << std::endl;
	}
#endif /* FULL_TEST */
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
        Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

void TestValid_bad_alloc( void )
{
    const std::string name("std::bad_alloc");

    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(false) << "Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	ThrowUnexpected<std::bad_alloc>();
      }
      catch(unexpected_exception& e)
      {
	Test.Check(true) << "Caught unexpected_exception(\""
			 << e.what() << "\", \""
			 << e.msg() << "\")"
			 << std::endl;
	
#if defined( FULL_TEST )
	Test.Check(std::string("std::")+std::string(e.what()) == name)
	  << "Names match - " << e.what() << " == " << name << std::endl;
#endif	/* FULL_TEST */
      }
      catch(...)
      {
        Test.Check(false) << "Caught wrong exception" << std::endl;
      }
    }
}

// Test with incorrect exception specification
void TestInvalid001()
    throw()
{
    throw std::runtime_error("Thrown from TestInvalid001");
}

int main(int argc, char** argv)
{
    Test.Init(argc, argv);

    LDASUnexpected::makeAbort( false );
    SignalHandler::Register( &abort_handler,
			     SignalHandler::SIGNAL_ABORT );

    if (!Test.IsVerbose())
    {
	LDASUnexpected::makeQuiet();
    }

    TestValid001();
    TestValid002();
    TestValid003();

#if FULL_TEST
#define TEST_VALID(exc)  TestValid<exc>(std::string(#exc), \
                                        std::string("Testing validity of " #exc))

#define TEST_VALID2(exc) TestValid<exc>(std::string(#exc))

    TEST_VALID(std::length_error);
    TEST_VALID(std::domain_error);
    TEST_VALID(std::out_of_range);
    TEST_VALID(std::invalid_argument);
    TEST_VALID(std::underflow_error);
    TEST_VALID(std::overflow_error);
    TEST_VALID(std::range_error);
    TEST_VALID(std::logic_error);
    TEST_VALID(std::runtime_error);

    TEST_VALID2(std::bad_alloc);
    TEST_VALID2(std::bad_exception);
    TEST_VALID2(std::bad_typeid);
    TEST_VALID2(std::bad_cast);
    TEST_VALID2(std::exception);

    TestValid_bad_alloc();
#endif	/* FULL_TEST */

#if ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER
    // This test unavoidably causes an abort, as it should
    // Setup to be silent
    int err_dup( dup( 2 ) );
    close( 2 );

    if ( setjmp(jmpbuffer) != 0 )
    {
	Test.Check(true) << "TestInvalid001 - Caught Abort Signal" << std::endl;
    }
    else
    {
      try {
	TestInvalid001();
      }
      catch(...)
      {
	// TestInvalid001 should cause an abort - if we catch
	// an exception in this case there is a bug!
	Test.Check(false) << "Caught an exception!"
			 << std::endl;
      }
    }
#endif	/* ! HAVE_NO_RETURN_FROM_UNEXPECTED_HANDLER */
    err_dup = dup( err_dup );

    Test.Exit();

    return 0;
}
